@@ -6,10 +6,11 @@ import json
 
 from pyot.pipeline.core import Pipeline
 from pyot.pipeline.token import PipelineToken
-from pyot.utils import ArrowCache
+from pyot.utils import PtrCache, camelcase, fast_copy
+from pyot.pipeline import pipelines
 
-normalizer_cache = ArrowCache()
-typing_cache = ArrowCache()
+normalizer_cache = PtrCache()
+typing_cache = PtrCache()
 
 
 class PyotLazyObject:
@@ -22,44 +23,25 @@ class PyotLazyObject:
                 self.clas = clas.__args__[0]
         except Exception:
             self.clas = clas
-        # server_map = {"server_type": [server_type, server]}
         self.server_map = [server_type, server]
-        # if isinstance(obj, list):
-        #     is_core = None
-        #     for l in obj:
-        #         if is_core is None:
-        #             is_core = issubclass(self.clas, PyotCoreObject)
-        #         if is_core:
-        #             l["data"].update(server_map)
-        #         else:
-        #             l.update(server_map)
-        # else:
-        #     if issubclass(self.clas, PyotCoreObject):
-        #         obj["data"].update(server_map)
-        #     else:
-        #         obj.update(server_map)
         self.obj = obj
 
     def __call__(self):
         if issubclass(self.clas, PyotCoreObject):
             if isinstance(self.obj, list):
-                l = []
+                li = []
                 for obj in self.obj:
-                    data = obj.pop("data")
-                    instance = self.clas(**obj)
-                    instance.meta.data = data
-                    instance.meta.server_map = self.server_map
+                    instance = self.clas()
+                    instance._meta.server_map = self.server_map
+                    instance._meta.data = instance._transform(obj)
                     instance._fill()
-                    l.append(instance)
-                return l
+                    li.append(instance)
+                return li
             else:
-                # pop the data
-                data = self.obj.pop("data")
-                # obj are the arguments
-                instance = self.clas(**self.obj)
-                # insert the data and fill
-                instance.meta.data = data
-                instance.meta.server_map = self.server_map
+                instance = self.clas()
+                # SERVER MAP WILL GO FIRST THAN OBJECT
+                instance._meta.server_map = self.server_map
+                instance._meta.data = instance._transform(self.obj)
                 instance._fill()
                 return instance
         elif issubclass(self.clas, PyotStaticObject):
@@ -67,12 +49,12 @@ class PyotLazyObject:
                 l = []
                 for obj in self.obj:
                     instance = self.clas(obj)
-                    instance.meta.server_map = self.server_map
+                    instance._meta.server_map = self.server_map
                     l.append(instance._fill())
                 return l
             else:
                 instance = self.clas(self.obj)
-                instance.meta.server_map = self.server_map
+                instance._meta.server_map = self.server_map
                 return instance._fill()
         raise RuntimeError(f"Unable to lazy load '{self.clas}'")
 
@@ -101,9 +83,9 @@ class PyotStaticObject:
 
     def __init__(self, data):
         # META CLASS UNIQUE MUTABLE OBJECTS
-        self.meta = self.Meta()
-        self.meta.data = data
-        self.meta.types = typing_cache.get(self.__class__, partial(get_type_hints, self.__class__))
+        self._meta = self.Meta()
+        self._meta.data = data
+        self._meta.types = typing_cache.get(self.__class__, partial(get_type_hints, self.__class__))
 
     def __getattribute__(self, name):
         try:
@@ -117,6 +99,9 @@ class PyotStaticObject:
         except (KeyError, AttributeError):
             raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
 
+    def __getitem__(self, item):
+        return self._meta.data[item]
+
     def _rename(self, data):
         # SNAKECASE > RENAME > REMOVE
         new_data = {}
@@ -124,10 +109,10 @@ class PyotStaticObject:
         for attr, val in data.items():
             name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', attr)
             newkey = re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()
-            if newkey in self.meta.removed:
+            if newkey in self._meta.removed:
                 continue
-            if newkey in self.meta.renamed:
-                newkey = self.meta.renamed[newkey]
+            if newkey in self._meta.renamed:
+                newkey = self._meta.renamed[newkey]
             mapping[attr] = newkey
             new_data[newkey] = val
 
@@ -143,35 +128,35 @@ class PyotStaticObject:
                 new_data, new_mapping = self._rename(data)
                 mapping.update(new_mapping)
                 return new_data
-        for key in self.meta.removed:
+        for key in self._meta.removed:
             new_data.pop(key, None)
         return new_data
 
     def _fill(self):
         # BIND SERVER > NORMALIZE > RAW > LAZY
         try:
-            server_type = self.meta.server_map
-            self.meta.server_type = server_type[0]
+            server_type = self._meta.server_map
+            self._meta.server_type = server_type[0]
             setattr(self, server_type[0], server_type[1].lower())
         except AttributeError: pass
 
-        data_ = self._normalize(self.meta.data)
+        data_ = self._normalize(self._meta.data)
 
-        if self.meta.server_type in data_:
+        if self._meta.server_type in data_:
             has_server = True
         else:
             has_server = False
 
         for attr, val in data_.items():
-            if attr in self.meta.raws:
+            if attr in self._meta.raws:
                 setattr(self, attr, val)
             elif PyotLazyObject.need_lazy(val):
                 if has_server:
-                    server = data_[self.meta.server_type]
+                    server = data_[self._meta.server_type]
                 else:
-                    server = getattr(self, self.meta.server_type)
-                setattr(self, attr, PyotLazyObject(self.meta.types[attr], val, self.meta.server_type, server))
-            elif attr == self.meta.server_type:
+                    server = getattr(self, self._meta.server_type)
+                setattr(self, attr, PyotLazyObject(self._meta.types[attr], val, self._meta.server_type, server))
+            elif attr == self._meta.server_type:
                 setattr(self, attr, val.lower())
             else:
                 setattr(self, attr, val)
@@ -184,12 +169,12 @@ class PyotStaticObject:
         Set `remove_server` to False to not remove the server values (region/platform/locale).
         '''
         if not pyotify:
-            return pickle.loads(pickle.dumps(self.meta.data)) # USING PICKLE FOR FASTER COPY
+            return fast_copy(self._meta.data) # USING PICKLE FOR FASTER COPY
         def recursive(obj):
             dic = obj.__dict__
-            del dic["meta"]
+            del dic["_meta"]
             if remove_server:
-                dic.pop(self.meta.server_type, None)
+                dic.pop(self._meta.server_type, None)
             for key, val in dic.items():
                 if type(val) is PyotLazyObject:
                     obj = val()
@@ -212,21 +197,65 @@ class PyotStaticObject:
         Set `remove_server` to False to not remove the server values (region/platform/locale).
         '''
         if not pyotify:
-            return json.dumps(self.meta.data)
+            return json.dumps(self._meta.data)
         return json.dumps(self.dict(pyotify=pyotify, remove_server=remove_server))
 
 
-class PyotCoreObject(PyotStaticObject):
+class PyotContainerObject:
+
+    class Meta:
+        server_type: str = "locale"
+        region_list = []
+        platform_list = []
+        locale_list = []
+
+    region: str = ""
+    platform: str = ""
+    locale: str = ""
+
+    def __init__(self, kwargs):
+        # META CLASS UNIQUE MUTABLE OBJECTS
+        self._meta = self.Meta()
+        self._set_server_type(kwargs)
+        for name, val in kwargs.items():
+            if name in ["platform", "region", "locale"] and val is not None:
+                setattr(self, name, val.lower())
+
+    def _get_server(self):
+        for server_type in ["platform", "region", "locale"]:
+            if self._meta.server_type == server_type:
+                server = getattr(self, server_type)
+                list_ = getattr(self._meta, server_type+"_list")
+                if server.lower() not in list_:
+                    raise ValueError(f"Invalid '{server_type}' value, '{server}' was given \
+                        {'. Did you activate the settings and set a default value ?' if not server else ''}")
+                self._meta.server = server.lower()
+                break
+            if server_type == "locale": # if server is last and still not found, raise
+                raise TypeError("Invalid or missing server type was passed as subclass")
+        return self
+
+    def _set_server_type(self, kwargs):
+        for server in ["platform", "region", "locale"]:
+            if server in kwargs:
+                self._meta.server_type = server
+                break
+            if server == "locale":  # if server is last and still not found, raise
+                raise TypeError("Invalid or missing server type was passed as subclass")
+
+
+class PyotCoreObject(PyotStaticObject, PyotContainerObject):
 
     class Meta(PyotStaticObject.Meta):
         # BE CAREFUL WHEN MANIPULATING MUTABLE OBJECTS
-        # ALL MUTABLE OBJECTS SHOULD BE OVERRIDDEN ON ITS SUBCLASS !
+        # ALL MUTABLE OBJECTS MUST BE OVERRIDDEN ON ITS SUBCLASS !
         pipeline: Pipeline
         key: str
         server: str
         session_id: str = None
         load: Mapping[str, Any]
         query: Mapping[str, Any]
+        body: Mapping[str, Any]
         server_type: str = "platform"
         allow_query: bool = False
         rules: Mapping[str, List[str]] = {}
@@ -234,51 +263,84 @@ class PyotCoreObject(PyotStaticObject):
         platform_list = []
         locale_list = []
 
-    async def get(self, sid: str = None):
+    async def get(self, sid: str = None, pipeline: str = None):
         '''Awaitable. Get this object from the pipeline.\n
-        `sid` may be passed to reuse a session on the pipeline.'''
+        `sid` id identifying the session on the pipeline to reuse.\n
+        `pipeline` key identifying the pipeline to execute against.\n
+        '''
+        self.set_pipeline(pipeline)
         token = await self.create_token()
-        data = await self.meta.pipeline.get(token, sid)
-        data = self.filter(data)
-        self.meta.data = self._transform(data)
+        data = await self._meta.pipeline.get(token, sid)
+        data = self._filter(data)
+        self._meta.data = self._transform(data)
+        self._fill()
+        return self
+
+    async def post(self, sid: str = None, pipeline: str = None):
+        '''Awaitable. Post this object to the pipeline.\n
+        `sid` id identifying the session on the pipeline to reuse.\n
+        `pipeline` key identifying the pipeline to execute against.\n
+        '''
+        self.set_pipeline(pipeline)
+        token = await self.create_token()
+        data = await self._meta.pipeline.post(token, self._meta.body, sid)
+        data = self._filter(data)
+        self._meta.data = self._transform(data)
+        self._fill()
+        return self
+
+    async def put(self, sid: str = None, pipeline: str = None):
+        '''Awaitable. Put this object to the pipeline.\n
+        `sid` id identifying the session on the pipeline to reuse.\n
+        `pipeline` key identifying the pipeline to execute against.\n
+        '''
+        self.set_pipeline(pipeline)
+        token = await self.create_token()
+        data = await self._meta.pipeline.put(token, self._meta.body, sid)
+        data = self._filter(data)
+        self._meta.data = self._transform(data)
         self._fill()
         return self
 
     def _lazy_set(self, kwargs):
         # META CLASS UNIQUE MUTABLE OBJECTS
-        self.meta = self.Meta()
-        self.meta.query = {}
-        self.meta.data = {}
-        self.meta.types = typing_cache.get(self.__class__, partial(get_type_hints, self.__class__))
+        self._meta = self.Meta()
+        self._meta.query = {}
+        self._meta.data = {}
+        self._meta.types = typing_cache.get(self.__class__, partial(get_type_hints, self.__class__))
 
-        for server in ["platform", "region", "locale"]:
-            if server in kwargs:
-                self.meta.server_type = server
-                break
-            if server == "locale":  # if server is last and still not found, raise
-                raise RuntimeError("Invalid or missing server type was passed as subclass")
+        self._set_server_type(kwargs)
         for name, val in kwargs.items():
             if name in ["platform", "region", "locale"] and val is not None:
-                self.meta.data[name] = val.lower()
+                self._meta.data[name] = val.lower()
                 setattr(self, name, val.lower())
             elif val is not None and name != "self":
-                self.meta.data[name] = val
+                self._meta.data[name] = val
                 setattr(self, name, val)
         return self
 
     def query(self, **kwargs):
         '''Add query parameters to the object.'''
-        if not self.meta.allow_query:
-            raise RuntimeError("This Pyot object does not accept queries")
-        self.meta.query = self._parse_query(locals())
+        self._meta.query = self._parse_camel(locals())
         return self
 
-    def to_camel_case(self, snake_str):
-        components = snake_str.split('_')
-        return components[0] + ''.join(x.title() for x in components[1:])
+    def body(self, **kwargs):
+        '''Add body parameters to the object.'''
+        self._meta.body = self._parse_camel(locals())
+        return self
 
-    def _parse_query(self, kwargs) -> Dict:
-        return {self.to_camel_case(key): val for (key,val) in kwargs.items() if key != "self" and val is not None}
+    def set_pipeline(self, pipeline: str = None):
+        '''Set the pipeline to execute against.'''
+        if pipeline is None: return self
+        try:
+            self._meta.pipeline = pipelines[pipeline]
+        except KeyError:
+            raise RuntimeError(f"Pipeline '{pipeline}' does not exist, inactive or dead")
+        return self
+
+    def _parse_camel(self, kwargs) -> Dict:
+        '''Parse locals to json compatible camelcased keys'''
+        return {camelcase(key): val for (key,val) in kwargs.items() if key != "self" and val is not None}
 
     async def create_token(self, search: str = None) -> PipelineToken:
         '''Awaitable. Create a pipeline token that identifies this object (its parameters).'''
@@ -286,13 +348,14 @@ class PyotCoreObject(PyotStaticObject):
         self._get_rule(search)
         self._get_server()
         self._refactor()
-        if not hasattr(self.meta, "pipeline"): raise RuntimeError("Pyot pipeline for this model wasn't activated or lost")
-        return PipelineToken(self.meta.pipeline.model, self.meta.server, self.meta.key, self.meta.load, self.meta.query)
+        self._validate()
+        if not hasattr(self._meta, "pipeline"): raise RuntimeError("Pyot pipeline for this model wasn't activated or lost")
+        return PipelineToken(self._meta.pipeline.model, self._meta.server, self._meta.key, self._meta.load, self._meta.query)
 
     def _get_rule(self, search):
-        if len(self.meta.rules) == 0:
+        if len(self._meta.rules) == 0:
             raise RuntimeError("This Pyot object is not getable")
-        for key, attr in self.meta.rules.items():
+        for key, attr in self._meta.rules.items():
             if search and search not in key: continue
             load = {}
             for a in attr:
@@ -302,32 +365,23 @@ class PyotCoreObject(PyotStaticObject):
                     break
             if len(load) != len(attr):
                 continue
-            self.meta.key = key
-            self.meta.load = load
+            self._meta.key = key
+            self._meta.load = load
             return self
-        raise ValueError("Incomplete values to create request token")
+        raise TypeError("Incomplete values to create request token")
 
-    def _get_server(self):
-        for server_type in ["platform", "region", "locale"]:
-            if self.meta.server_type == server_type:
-                server = getattr(self, server_type)
-                list_ = getattr(self.meta, server_type+"_list")
-                if server.lower() not in list_:
-                    raise ValueError(f"Invalid '{server_type}' value, '{server}' was given")
-                self.meta.server = server.lower()
-                break
-            if server_type == "locale": # if server is last and still not found, raise
-                raise RuntimeError("Invalid or missing server type was passed as subclass")
-        return self
 
     async def _clean(self):
         pass
 
+    def _validate(self):
+        pass
+
     def _transform(self, data) -> Dict:
         return data
 
     def _refactor(self):
         pass
 
-    def filter(self, data):
+    def _filter(self, data):
         return data
