@@ -9,6 +9,8 @@ from .token import PipelineToken
 
 
 class Pipeline:
+    '''Pyot Pipeline object, backbone of the entire framework.'''
+
     def __init__(self, model: str, stores: List[Any]):
         self.model = model
         self.stores = stores
@@ -16,9 +18,8 @@ class Pipeline:
 
     async def get(self, token: PipelineToken, sid: str = None):
         '''Get an object from the stores.'''
-        session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) if sid is None else self.sessions[sid]
-        found_in = None
-        try:
+        async with PipelineSession(self, sid) as session:
+            found_in = None
             for store in self.stores:
                 try:
                     response = await store.get(token, session=session)
@@ -28,14 +29,7 @@ class Pipeline:
                         raise
                     continue
                 break
-        except:
-            if sid is None: await session.close()
-            raise
-        finally:
-            if sid is None: await session.close()
-
         await self.set(token, response, found_in)
-
         return response
 
     async def set(self, token: PipelineToken, value: Any, stop=None):
@@ -47,13 +41,31 @@ class Pipeline:
             except NotImplementedError:
                 continue
 
-    async def post(self, token: PipelineToken, value: Any):
+    async def post(self, token: PipelineToken, body: Any, sid: str = None):
         '''Post an object to stores of type Service.'''
-        for store in self.stores:
-            try:
-                await store.post(token, value)
-            except NotImplementedError:
-                continue
+        async with PipelineSession(self, sid) as session:
+            for store in self.stores:
+                try:
+                    response = await store.post(token, body, session=session)
+                except (NotImplementedError, NotFound, NotFindable) as e:
+                    if self.stores[-1] == store or store.store_type == "SERVICE" and not isinstance(e, NotFindable):
+                        raise
+                    continue
+                break
+        return response
+
+    async def put(self, token: PipelineToken, body: Any, sid: str = None):
+        '''Put an object to stores of type Service.'''
+        async with PipelineSession(self, sid) as session:
+            for store in self.stores:
+                try:
+                    response = await store.put(token, body, session=session)
+                except (NotImplementedError, NotFound, NotFindable) as e:
+                    if self.stores[-1] == store or store.store_type == "SERVICE" and not isinstance(e, NotFindable):
+                        raise
+                    continue
+                break
+        return response
 
     async def clear(self):
         '''Clear stores of type Cache.'''
@@ -89,3 +101,20 @@ class Pipeline:
                 continue
             if contains: break
         return contains
+
+
+class PipelineSession:
+    '''Manages a Pipeline Session'''
+    session: aiohttp.ClientSession
+
+    def __init__(self, pipeline: Pipeline, sid: str):
+        self.pipeline = pipeline
+        self.sid = sid
+
+    async def __aenter__(self) -> "PipelineSessionManager":
+        self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) if self.sid is None else self.pipeline.sessions[self.sid]
+        return self.session
+
+    async def __aexit__(self, *args):
+        if self.sid is None:
+            await self.session.close()
