@@ -8,3 +8,5 @@ from .rediscache import RedisCache
 from .diskcache import DiskCache
 from .merakicdn import MerakiCDN
 from .cdragon import CDragon
+from .mongodb import MongoDB
+from .ddragon import DDragon
