@@ -1,8 +1,9 @@
 from .champion import champion_id_by_key, champion_id_by_name, champion_key_by_id, champion_key_by_name, champion_name_by_id, \
     champion_name_by_key, _utils_inner_cache
 from .cdragon import cdragon_url, cdragon_sanitize, tft_url, tft_item_sanitize, tft_champ_sanitize
-from .objects import arrow_cache, clone_generator, ArrowCache, CloneGenerator
+from .objects import ptr_cache, frozen_generator, PtrCache, FrozenGenerator
 from .dicts import multi_defaultdict, redis_defaultdict, MultiDefaultDict, RedisDefaultDict
 from .common import snakecase, camelcase, shuffle_list, loop_run, thread_run, import_class, fast_copy, pytify, bytify
 from .locks import SealLock, RedisLock
 from .time import timeit, atimeit
+from .lor import batch_to_ccac
